/**
 ******************************************************************************
 * @file    Project/STM32F0xx_StdPeriph_Templates/stm32f0xx_it.c 
 * @author  MCD Application Team
 * @version V1.3.1
 * @date    17-January-2014
 * @brief   Main Interrupt Service Routines.
 *          This file provides template for all exceptions handler and 
 *          peripherals interrupt service routine.
 ******************************************************************************
 * @attention
 *
 * <h2><center>&copy; COPYRIGHT 2014 STMicroelectronics</center></h2>
 *
 * Licensed under MCD-ST Liberty SW License Agreement V2, (the "License");
 * You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at:
 *
 *        http://www.st.com/software_license_agreement_liberty_v2
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include <stdint.h>
#include <stdio.h>
#include <string.h>

#include "stm32f0xx_conf.h"
#include "stm32f0xx_it.h"
#include "main.h"
#include "ringbuffer.h"
#include "delay.h"

#include "dbgprintf.h"


/** @addtogroup Template_Project
 * @{
 */

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

volatile uint32_t syscnt = 0;

volatile uint16_t prevCapture = 0;
extern ringbuffer_t *pRingbuffer;

/******************************************************************************/
/*            Cortex-M0 Processor Exceptions Handlers                         */
/******************************************************************************/

/**
 * @brief  This function handles NMI exception.
 * @param  None
 * @retval None
 */
void NMI_Handler(void)
{
}

/**
 * @brief  This function handles Hard Fault exception.
 * @param  None
 * @retval None
 */
void HardFault_Handler(void)
{
   /* Go to infinite loop when Hard Fault exception occurs */
   while (1)
   {
   }
}

/**
 * @brief  This function handles SVCall exception.
 * @param  None
 * @retval None
 */
void SVC_Handler(void)
{
}

/**
 * @brief  This function handles PendSVC exception.
 * @param  None
 * @retval None
 */
void PendSV_Handler(void)
{
}

/**
 * @brief  This function handles SysTick Handler.
 * @param  None
 * @retval None
 */
void SysTick_Handler(void)
{
   syscnt++;
//   if (syscnt % 2)
//      sbit(GPIOA, 5);
//   else
//      cbit(GPIOA, 5);
}

/******************************************************************************/
/*                 STM32F0xx Peripherals Interrupt Handlers                   */
/*  Add here the Interrupt Handler for the used peripheral(s) (PPP), for the  */
/*  available peripheral interrupt handler's name please refer to the startup */
/*  file (startup_stm32f0xx.s).                                            */
/******************************************************************************/



/**
 * @brief  This function handles TIM3 global interrupt request.
 * @param  None
 * @retval None
 */
void TIM3_IRQHandler(void)
{
   if (TIM_GetITStatus(TIM3, TIM_IT_Update) != RESET)
   {
      TIM_ClearITPendingBit(TIM3, TIM_IT_Update);

      TIM_ClearFlag(TIM3, TIM_FLAG_Update | TIM_FLAG_CC1 | TIM_FLAG_CC2 | TIM_FLAG_CC3 | TIM_FLAG_CC4);
   }

   /* TIM3_CH1 */
   if (TIM_GetITStatus(TIM3, TIM_IT_CC1) != RESET)
   {
      TIM_ClearITPendingBit(TIM3, TIM_IT_CC1 );
   }

   /* TIM3_CH2 */
   if (TIM_GetITStatus(TIM3, TIM_IT_CC2) != RESET)
   {
      TIM_ClearITPendingBit(TIM3, TIM_IT_CC2);
   }

   /* TIM3_CH3 */
   if (TIM_GetITStatus(TIM3, TIM_IT_CC3) != RESET)
   {
      TIM_ClearITPendingBit(TIM3, TIM_IT_CC3);
   }

   /* TIM3_CH4 */
   if (TIM_GetITStatus(TIM3, TIM_IT_CC4) != RESET)
   {
      TIM_ClearITPendingBit(TIM3, TIM_IT_CC4);
   }
}



/**
  * @brief  This function handles External lines 4 to 15 interrupt request.
  * @param  None
  * @retval None
  */
void EXTI4_15_IRQHandler(void)
{
   if(EXTI_GetITStatus(EXTI_Line4) != RESET)
   {
      /* Clear the EXTI line 4 pending bit */
      EXTI_ClearITPendingBit(EXTI_Line4);
   }
}



void TIM14_IRQHandler(void)
{
   uint16_t capture;
   uint16_t diff;
   uint8_t *p = (uint8_t *)&diff;

   if(TIM_GetITStatus(TIM14, TIM_IT_CC1) == SET)
   {
      sbit(GPIOA, 5);

      /* Clear TIM14 Capture compare interrupt pending bit */
      TIM_ClearITPendingBit(TIM14, TIM_IT_CC1);

      capture = TIM_GetCapture1(TIM14);
      diff = capture - prevCapture;
#if 01
      if (ringbuffer_available(pRingbuffer) > 2)
      {
         ringbuffer_write(pRingbuffer, p, 2);
      }
      else
      {
         dbgprintf(("Out of buffer\r\n"));
      }
#endif
      prevCapture = capture;

      cbit(GPIOA, 5);
   }

}

/**
 * @brief  This function handles PPP interrupt request.
 * @param  None
 * @retval None
 */
/*void PPP_IRQHandler(void)
{
}*/

/**
 * @}
 */ 


/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
